//--------------------------------------------------------------------
//	
//	ToolTipDemo.cpp
//
//  Written by Robert Polic
//
//--------------------------------------------------------------------

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ToolTipDemo.h"


//====================================================================

main()
{	
	ToolTipDemoApp	app;

	app.Run();
	return(B_NO_ERROR);
}


//====================================================================

ToolTipDemoApp::ToolTipDemoApp()
			   :BApplication("application/x-vnd.Be-Help"),
			    fPrefWindow(NULL)
{
	// set up a rectangle, instantiate and show a new window
	(new ToolTipDemoWindow(BRect(100, 80, 300, 152)))->Show();
	// instantiate the tooltip window but don't show it
	fToolTip = new TToolTip();
}

//--------------------------------------------------------------------

void ToolTipDemoApp::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		// these messages need to be forwarded to the TToolTip class
		case B_SOME_APP_ACTIVATED:
		case eToolTipStart:
		case eToolTipStop:
			fToolTip->PostMessage(msg);
			break;

		// message from main window to bring up ToolTip settings panel
		case 'PREF':
			if (!fPrefWindow)
			{
				tool_tip_settings	settings;

				// get the current ToolTip settings
				fToolTip->GetSettings(&settings);
				// open the settings panel
				(fPrefWindow = new ToolTipPrefsWindow(BRect(140, 120, 340, 320), &settings))->Show();
			}
			break;
		// message received from settings panel that it closed
		case 'PCLS':
			fPrefWindow = NULL;
			break;

		case 'SET ':
			{
				tool_tip_settings	*settings;
				ssize_t				len;

				// forward new settings to ToolTip window
				msg->FindData("settings", B_RAW_TYPE, (const void **)&settings, &len);
				fToolTip->SetSettings(settings);
			}
			break;
	}
}


//====================================================================
//
// Demo window that creates a few controls with ToolTips
//
//====================================================================

ToolTipDemoWindow::ToolTipDemoWindow(BRect frame)
				  :BWindow(frame, "Tool-Tip Demo", B_TITLED_WINDOW, B_NOT_RESIZABLE |
				  													B_NOT_ZOOMABLE)
{
	BView			*view;
	TToolTipButton	*button;

	// set up a background view the size of the window
	AddChild(view = new BView(Bounds(), "view", B_FOLLOW_ALL, 0));
	view->SetViewColor(144, 176, 160, 255);

	// add controls to the window
	view->AddChild(new TToolTipTextControl(BRect(10, 10, 189, 10), "tool_tip_text",
		"Tip:", "Enter tip to be displayed here", new BMessage('TEXT'), ""));
	view->AddChild(new TToolTipButton(BRect(10, 44, 90, 44), "tool_tip_button",
		"Quit", new BMessage(B_QUIT_REQUESTED), "Click this button to quit"));
	view->AddChild(button = new TToolTipButton(BRect(110, 44, 190, 44), "tool_tip_button",
		"Preferences", new BMessage('PREF'), "Click this button to set ToolTip preferences"));
	button->SetTarget(be_app);
}

//--------------------------------------------------------------------

bool ToolTipDemoWindow::QuitRequested(void)
{
	// tell app to quit
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}	


//====================================================================
//
// Settings panel for ToolTip parameters
//
//====================================================================

ToolTipPrefsWindow::ToolTipPrefsWindow(BRect frame, tool_tip_settings *settings)
				   :BWindow(frame, "Tool-Tip Preferences", B_MODAL_WINDOW,
							B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BView				*background;
	TToolTipButton		*button;
	TToolTipCheckBox	*checkbox;

	fOriginalSettings = fCurrentSettings = *settings;

	// set up a background view the size of the window
	background = new BView(Bounds(), "view", B_FOLLOW_ALL, 0);
	AddChild(background);
	background->SetViewColor(224, 224, 224, 255);

	// add controls to the window
	fDelay = new TToolTipSlider(BRect(10, 10, 189, 10), "tool_tip_slider",
		"", new BMessage('DLAY'), 0, 40, "Set delay before ToolTip is displayed");
	background->AddChild(fDelay);
	fDelay->SetValue(fOriginalSettings.delay / 50000);
	SetDelayLabel(fOriginalSettings.delay / 50000);

	fHold = new TToolTipSlider(BRect(10, 50, 189, 50), "tool_tip_slider",
		"", new BMessage('DURA'), 0, 60, "Set length that ToolTip is displayed");
	background->AddChild(fHold);
	fHold->SetValue(fOriginalSettings.hold / 1000000);
	SetHoldLabel(fOriginalSettings.hold / 1000000);

	fFontSize = new TToolTipSlider(BRect(10, 90, 189, 90), "tool_tip_slider",
		"", new BMessage('FSIZ'), 5, 127, "Set the ToolTip font size");
	background->AddChild(fFontSize);
	fFontSize->SetValue((int)fOriginalSettings.font.Size());
	SetFontSizeLabel(fOriginalSettings.font.Size());

	checkbox = new TToolTipCheckBox(BRect(10, 140, 120, 140), "tool_tip_button",
		"Disable ToolTips", new BMessage('TIPS'), "Click to disable ToolTips");
	background->AddChild(checkbox);
	checkbox->SetValue(!fOriginalSettings.enabled);

	button = new TToolTipButton(BRect(10, 170, 90, 170), "tool_tip_button",
		"Cancel", new BMessage(B_CANCEL), "Click this button to cancel changes");
	background->AddChild(button);

	button = new TToolTipButton(BRect(110, 170, 190, 170), "tool_tip_button",
		"OK", new BMessage((unsigned long) B_OK), "Click this button to apply changes");
	background->AddChild(button);
	button->MakeDefault(true);
}

//--------------------------------------------------------------------

ToolTipPrefsWindow::~ToolTipPrefsWindow()
{
	be_app->PostMessage('PCLS');
}

//--------------------------------------------------------------------

void ToolTipPrefsWindow::MessageReceived(BMessage *msg)
{
	bool		close = false;
	bool		update = false;
	int32		value;

	switch (msg->what) {
		case B_CANCEL:
			fCurrentSettings = fOriginalSettings;
			update = true;
			close = true;
			break;
		case B_OK:
			close = true;
			break;
		case 'DLAY':
			msg->FindInt32("be:value", &value);
			fCurrentSettings.delay = value * 50000;
			SetDelayLabel(value);
			update = true;
			break;
		case 'DURA':
			msg->FindInt32("be:value", &value);
			fCurrentSettings.hold = value * 1000000;
			SetHoldLabel(value);
			update = true;
			break;
		case 'FSIZ':
			msg->FindInt32("be:value", &value);
			fCurrentSettings.font.SetSize(value);
			SetFontSizeLabel(value);
			update = true;
			break;
		case 'TIPS':
			msg->FindInt32("be:value", &value);
			fCurrentSettings.enabled = !value;
			update = true;
			break;
		default:
			BWindow::MessageReceived(msg);
	}

	if (update) {
		BMessage	m;

		m.what = 'SET ';
		m.AddData("settings", B_RAW_TYPE, &fCurrentSettings, sizeof(tool_tip_settings));
		be_app->PostMessage(&m);
	}

	if (close)
		Close();
}

//--------------------------------------------------------------------

void ToolTipPrefsWindow::SetDelayLabel(int32 value)
{
	char		label[128];

	sprintf(label, "ToolTip Delay: %.2f secs", (float)((value * 5.0) / 100.0));
	fDelay->SetLabel(label);
}

//--------------------------------------------------------------------

void ToolTipPrefsWindow::SetFontSizeLabel(float value)
{
	char		label[128];

	sprintf(label, "ToolTip Font Size: %d", (int)value);
	fFontSize->SetLabel(label);
}

//--------------------------------------------------------------------

void ToolTipPrefsWindow::SetHoldLabel(int32 value)
{
	char		label[128];

	sprintf(label, "ToolTip Duration: %d secs", (int)value);
	fHold->SetLabel(label);
}
